@@ -17,20 +17,20 @@ FCGI::Buffer - Verify, Cache and Optimise FCGI Output
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 =head1 SYNOPSIS
 
 FCGI::Buffer verifies the HTML that you produce by passing it through
 C<HTML::Lint>.
 
-FCGI::Buffer optimises FCGI programs by compressing output to speed up
-the transmission and by nearly seamlessly making use of client and
-server caches.
+FCGI::Buffer optimises FCGI programs by reducing, filtering and compressing
+output to speed up the transmission and by nearly seamlessly making use of
+client and server caches.
 
 To make use of client caches, that is to say to reduce needless calls
 to your server asking for the same data:
@@ -132,6 +132,10 @@ sub DESTROY {
 
 	select($self->{old_buf});
 	if(!defined($self->{buf})) {
+		# Unlikely
+		if($self->{'logger'}) {
+			$self->{'logger'}->info('Nothing to send');
+		}
 		return;
 	}
 	my $pos = $self->{buf}->getpos;
@@ -148,12 +152,14 @@ sub DESTROY {
 		if($ENV{'HTTP_IF_MODIFIED_SINCE'}) {
 			$self->{logger}->debug("HTTP_IF_MODIFIED_SINCE: $ENV{HTTP_IF_MODIFIED_SINCE}");
 		}
-		$self->{logger}->debug("Generate_etag = $self->{generate_etag}");
-		$self->{logger}->debug("Generate_304 = $self->{generate_304}");
-		$self->{logger}->debug("Generate_last_modified = $self->{generate_last_modified}");
+		$self->{logger}->debug("Generate_etag = $self->{generate_etag}",
+			"Generate_304 = $self->{generate_304}",
+			"Generate_last_modified = $self->{generate_last_modified}");
 	}
 	unless($headers || $self->is_cached()) {
-		# There was no output
+		if($self->{'logger'}) {
+			$self->{'logger'}->debug('There was no output');
+		}
 		return;
 	}
 	if($ENV{'REQUEST_METHOD'} && ($ENV{'REQUEST_METHOD'} eq 'HEAD')) {
@@ -549,14 +555,7 @@ sub DESTROY {
 		# FCGI::Buffer
 		unshift @{$self->{o}}, split(/\r\n/, $headers);
 		if($self->{body} && $self->{send_body}) {
-			my $already_done = 0;
-			foreach(@{$self->{o}}) {
-				if(/^Content-Length: /) {
-					$already_done = 1;
-					last;
-				}
-			}
-			unless($already_done) {
+			unless(grep(/^Content-Length: /, $self->{o})) {
 				push @{$self->{o}}, "Content-Length: $body_length";
 			}
 		}
@@ -661,6 +660,7 @@ sub _check_modified_since {
 	}
 }
 
+# Reduce output, e.g. remove superfluous white-space.
 sub _optimise_content {
 	my $self = shift;
 
@@ -786,7 +786,7 @@ Set various options and override default values.
 	cache_key => 'string',	# key for the cache
 	cache_age => '10 minutes',	# how long to store responses in the cache
 	logger => $self->{logger},
-	lint->content => 0,	# Pass through HTML::Lint
+	lint_content => 0,	# Pass through HTML::Lint
 	generate_304 => 1,	# Generate 304: Not modified
     );
 
@@ -858,7 +858,8 @@ sub init {
 	my $pos = $self->{buf}->getpos;
 	if($pos > 0) {
 		if(defined($self->{logger})) {
-			$self->{logger}->warn("Too late to call init, $pos characters have been printed");
+			my @call_details = caller(0);
+			$self->{logger}->warn("Too late to call init, $pos characters have been printed, caller line $call_details[2] of $call_details[1]");
 		} else {
 			# Must do Carp::carp instead of carp for Test::Carp
 			Carp::carp "Too late to call init, $pos characters have been printed";
@@ -1086,7 +1087,7 @@ sub _should_gzip {
 
 	if($self->{compress_content} && ($ENV{'HTTP_ACCEPT_ENCODING'} || $ENV{'HTTP_TE'})) {
 		my $accept = lc($ENV{'HTTP_ACCEPT_ENCODING'} ? $ENV{'HTTP_ACCEPT_ENCODING'} : $ENV{'HTTP_TE'});
-		foreach my $encoding ('x-gzip', 'gzip') {
+		foreach my $encoding ('x-gzip', 'gzip', 'br') {
 			$_ = $accept;
 			if(defined($self->{content_type})) {
 				my @content_type = @{$self->{content_type}};
@@ -1131,19 +1132,37 @@ sub _compress()
 	if((length($encoding) == 0) || (length($self->{body}) < MIN_GZIP_LEN)) {
 		return;
 	}
-	require Compress::Zlib;
-	Compress::Zlib->import;
 
-	# Avoid 'Wide character in memGzip'
-	unless($self->{_encode_loaded}) {
-		require Encode;
-		$self->{_encode_loaded} = 1;
-	}
-	my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($self->{body}));
-	if(length($nbody) < length($self->{body})) {
-		$self->{body} = $nbody;
-		push @{$self->{o}}, "Content-Encoding: $encoding";
-		push @{$self->{o}}, "Vary: Accept-Encoding";
+	if($encoding eq 'gzip') {
+		require Compress::Zlib;
+		Compress::Zlib->import;
+
+		# Avoid 'Wide character in memGzip'
+		unless($self->{_encode_loaded}) {
+			require Encode;
+			$self->{_encode_loaded} = 1;
+		}
+		my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($self->{body}));
+		if(length($nbody) < length($self->{body})) {
+			$self->{body} = $nbody;
+			push @{$self->{o}}, "Content-Encoding: $encoding";
+			push @{$self->{o}}, "Vary: Accept-Encoding";
+		}
+	} elsif($encoding eq 'br') {
+		require IO::Compress::Brotli;
+		IO::Compress::Brotli->import();
+
+		# Avoid 'Wide character in memGzip'
+		unless($self->{_encode_loaded}) {
+			require Encode;
+			$self->{_encode_loaded} = 1;
+		}
+		my $nbody = bro(Encode::encode_utf8($self->{body}));
+		if(length($nbody) < length($self->{body})) {
+			$self->{body} = $nbody;
+			push @{$self->{o}}, "Content-Encoding: $encoding";
+			push @{$self->{o}}, "Vary: Accept-Encoding";
+		}
 	}
 }
 
@@ -1274,7 +1293,7 @@ The licence for cgi_buffer is:
 
     This software is provided 'as is' without warranty of any kind."
 
-The rest of the program is Copyright 2015 Nigel Horne,
+The rest of the program is Copyright 2015-2016 Nigel Horne,
 and is released under the following licence: GPL
 
 =cut
